@@ -9,7 +9,8 @@ def clear_screen():
 if __name__ == '__main__':
     while True:
         session_token = input('Please enter your session token: ')
-        chat = ChatGPT(session_token)
+        conversation_id = input('Please enter your conversation id (if you want to continue old chat): ')
+        chat = ChatGPT(session_token,conversation_id)
         break
 
     clear_screen()
@@ -17,7 +18,7 @@ if __name__ == '__main__':
         'Conversation started. Type "reset" to reset the conversation. Type "quit" to quit.\n'
     )
     while True:
-        prompt = input('You: ')
+        prompt = input('\nYou: ')
         if prompt.lower() == 'reset':
             chat.reset_conversation()
             clear_screen()
@@ -26,7 +27,6 @@ if __name__ == '__main__':
             )
             continue
         if prompt.lower() == 'quit':
-            chat.close()
             break
         print('\nChatGPT: ', end='')
         response = chat.send_message(prompt)
